local Game = {}

local insert = table.insert

function Game:reinit()
	self:reinitgameloop()
	self.maxobjects = 1
	self.objects = {}
	self.gameloop = {}
	self.scene = "main"
	self.sceneargs = {}
	self.gravity = 800
	self.sounds = require "sounds"
	self.currentvolume = 1
	self.bgmvolume = 1
	self:setVolume(1)
	self.scenes = require "scenes"
	self.objects.base = require "objects/base"
	self.objects.warp = require "objects/warp"
	self.objects.apple = require "objects/enemies/apple"
	self.objects.carrot = require "objects/carrot"
	self.objects.background = require "objects/background"
	self.objects.transition = require "objects/transition"
	self.objects.coin = require "objects/coin"
	self.objects.balcony = require "objects/balcony"
	self.objects.slidetransition = require "objects/slidetransition"
	self.objects.movable = require "objects/movable"
	self.objects.water = require "objects/water"
	self.objects.player = require "objects/player"
	self.objects.tileset = require "objects/tileset"
	self.objects.particle = require "objects/particle"
	self.objects.squid = require "objects/enemies/squid"
	self.objects.movingplatform = require "objects/movingplatform"
	self.objects.fallingplatform= require "objects/fallingplatform"
	self.objects.spring = require "objects/spring"
	self.objects.bug = require "objects/enemies/bug"
	self.objects.breakable = require "objects/breakable"
	self.objects.flower = require "objects/flower"
	self.objects.flowerpot = require "objects/flowerpot"
	self.objects.mushroom = require "objects/mushroom"
	self.objects.enemy = require "objects/enemies/enemy"
	self.objects.bird = require "objects/enemies/bird"
	self.objects.endflag = require "objects/endflag"
	self.objects.sign = require "objects/sign"
	self.objects.door = require "objects/door"
	self.objects.car = require "objects/car"
	self.rect = require "lib/rect"
	self.shaders = require "lib/shaders"
	self:setLang("en")
	self.camerax = 0
	self.debug = false
	self.cameray = 0
	self.floatcamerax = 0
	self.floatcameray = 0
	self.assets = {} -- Caching loaded images
	self.particles = 1 -- Changing the number of particles
	self.maps = {} -- Caching loaded maps
	self:initInputs()
	love.window.setIcon(love.image.newImageData("assets/icon.png"))
end

function Game:setLang(lang)
	local file = "text/"..lang
	self.texts = require(file)
	self.lang = lang
end

function Game:reinitgameloop()
	self.gameloop = {}
end

function Game:changesize(width,height)
		self.WIDTH = width
		self.HEIGHT = height
end

function Game:movecamera(movex,movey)
	self.floatcamerax = self.floatcamerax + movex
	self.floatcameray = self.floatcameray + movey
	self.camerax = math.floor(self.floatcamerax+0.5)
	self.cameray = math.floor(self.floatcameray+0.5)
end
	
function Game:cameraclamp(minx,miny,maxx,maxy)
	self.floatcamerax = math.max(minx,math.min(maxx-self.WIDTH,self.floatcamerax))
	self.floatcameray = math.max(miny,math.min(maxy-self.HEIGHT,self.floatcameray))
	self.camerax = math.max(minx,math.min(maxx-self.WIDTH,self.camerax))
	self.cameray = math.max(miny,math.min(maxy-self.HEIGHT,self.cameray))
end

function Game:setcamera(x,y)
	self.floatcamerax = x
	self.floatcameray = y
	self.camerax = math.floor(self.floatcamerax+0.5)
	self.cameray = math.floor(self.floatcameray+0.5)
end

function Game:initInputs()
	self.inputs = {}
	self.inputs.touch = {}
	self.inputs.touch.move = {id=nil}
	self.inputs.touch.jump = {id=nil}
	self.inputs.deadzone = 0.8
	self.inputs.keys = {}
	self.inputs.buttons = {}
	self.inputs.axis= {}
	self.inputs.actions = {}
	for i,v in pairs({"up","down","left","right","menu","jump","stomp"}) do
		local action = {}
		action.keys = {}
		action.buttons = {}
		action.axis = {}
		self.inputs.actions[v] = action
		action.timer = 0
		action.realtimer = 0
		action.lastpress = 99
	end
	self.inputs.actions.jump.keys = {"space","z","j"}
	self.inputs.actions.jump.buttons = {"a","b"}
	self.inputs.actions.stomp.keys = {"x","k"}
	self.inputs.actions.stomp.buttons = {"x","y"}
	self.inputs.actions.up.keys = {"up","w"}
	self.inputs.actions.up.buttons = {"dpup"}
	self.inputs.actions.up.axis = {{"lefty",-1}}
	self.inputs.actions.down.keys = {"down","s"}
	self.inputs.actions.down.buttons = {"dpdown"}
	self.inputs.actions.down.axis = {{"lefty",1}}
	self.inputs.actions.left.keys = {"left","a"}
	self.inputs.actions.left.buttons = {"dpleft"}
	self.inputs.actions.left.axis = {{"leftx",-1}}
	self.inputs.actions.right.keys = {"right","d"}
	self.inputs.actions.right.buttons = {"dpright"}
	self.inputs.actions.right.axis = {{"leftx",1}}
	self.inputs.actions.menu.keys = {"escape","return"}
	self.inputs.actions.menu.buttons = {"start","back"}
end

function Game:updateInputs(dt)
	for i,action in pairs(self.inputs.actions) do
		local pressed = false
		for _,key in pairs(action.keys) do
			if self.inputs.keys[key] then pressed = true end
		end
		for _,button in pairs(action.buttons) do
			if self.inputs.buttons[button] then pressed = true end
		end
		for _,axis in pairs(action.axis) do
			if self.inputs.axis[axis[1]]~=nil then
				if self.inputs.axis[axis[1]]*axis[2]>self.inputs.deadzone then
					pressed = true
				end
			end
		end

		if pressed then
			action.realtimer = action.realtimer+dt
			action.timer = action.timer+1
			if action.timer == 1 then
				action.lastpress = 0
			end
		else
			action.timer = 0
			action.realtimer = 0
		end
		action.lastpress = action.lastpress + dt
	end
end

function Game:summon(obj)
	obj.game = self
	obj.id = self.maxobjects
	self.maxobjects = self.maxobjects + 1
	self.gameloop[obj.id] = obj
end

function Game:findName(askedname)
	local result = {}
	for i,v in pairs(self.gameloop) do
		if v~=nil then
			for j,name in ipairs(v.classes) do
				if name==askedname then
					insert(result,v)
				end
			end
		end
	end
	return result
end

function Game:delid(objid)
	self.gameloop[objid] = nil
end

function Game:delname(objName)
	for i,v in ipairs(self:findName(objName)) do
		if v~=nil then
			Game:delid(objid)
		end
	end
end

function Game:step(dt)
	self:updateInputs(dt)
	print(#self.gameloop)
	for i,v in pairs(self.gameloop) do
		--Processing all the objects
		print("	"..v.classes[#v.classes])
		if v~=nil then v:step(dt) end
	end
end

function Game:newImage(path)
	local image = nil
	if self.assets[path]~=nil then -- If already loaded
		image = self.assets[path]
	else
		image = love.graphics.newImage("assets/"..path) -- Load it and save it
		self.assets[path] = image
	end
	return image
end

function Game:getMap(name)
	local map = nil
	if self.maps[name]~=nil then
		map = self.maps[name]
	else
		map = require ("maps/"..name)
		self.maps[name] = map
	end
	return map
end

function Game:getSpriteDir(path,ext,func,folder)
	if not ext then ext = ".png" end
	if not func then func = self.newImage end
	if not folder then folder = "assets/" end
	local counter = 0
	local result = {}
	while love.filesystem.getInfo(folder..path..counter..ext) do
		print(folder..path..counter..ext)
		insert(result,func(self,path..counter..ext))
		counter = counter + 1
	end
	return result
end

function Game:getLevel(i)
	local level = nil
	if self.levels[i]~=nil then
		level = self.levels[i]
	else
		level = {}
		level.folder = "maps/Level "..i.."/"
		local module = level.folder.."data"
		level.data = require(module)
		level.backgrounds = {}
		level.bgsizes = {}
		-- Get backgrounds
		local counter = 0
		while love.filesystem.getInfo(level.folder.."background"..counter..".png") do
			print(level.folder.."background"..counter..".png")
			img = love.graphics.newImage(level.folder.."background"..counter..".png")
			table.insert(level.backgrounds,img)
			table.insert(level.bgsizes,{img:getWidth(),img:getHeight()})
			counter = counter + 1
		end
		self.levels[i] = level
	end
	return level
end

function Game:draw(screen)
	draworder = {}
	for i,v in pairs(self.gameloop) do
		if v~=nil then
			table.insert(draworder,v)
		end
	end
	table.sort(draworder, function(a,b) return a.depth< b.depth end)
	for i,v in ipairs(draworder) do
		--Processing all the objects
		if v~=nil then
			if screen~= "bottom" then
				if v.draw ~= nil then
					local offset = 0
					if screen=="left" then offset = -love.graphics.get3DDepth() end
					if screen=="right" then offset = love.graphics.get3DDepth() end
					self.camerax = self.camerax + offset
					v:draw()
					self.camerax = self.camerax - offset
				end
			else
				if v.bottomdraw~=nil then v.bottomdraw() end
			end
		end
	end
	if self.debug then lg.print("OSTYPE: "..self.OSTYPE.."; ".."OS: "..self.OS.."; ".."FPS: "..tostring(love.timer.getFPS( )), 10, 10) end
end

function Game:MouseCallback(x,y,id)
	x = (x-self.DISPLAYOFFX)/self.WIDTH
	y = (y-self.DISPLAYOFFY)/self.HEIGHT
	if x<0.5 then
		self.inputs.touch.move = {}
		self.inputs.touch.move.id = id
		self.inputs.touch.move.x = x
		self.inputs.touch.move.y = y
	else
		self.inputs.touch.jump = {}
		self.inputs.touch.jump.id = id
		self.inputs.touch.jump.x = x
		self.inputs.touch.jump.y = y
	end
end
function Game:ReleaseCallback(x,y,id)
	x = (x-self.DISPLAYOFFX)/self.WIDTH
	y = (y-self.DISPLAYOFFY)/self.HEIGHT
	if x<0.5 then
		if self.inputs.touch.move.id == id then
			self.inputs.touch.move.id = nil
		end
	else
		if self.inputs.touch.jump.id == id then
			self.inputs.touch.jump.id = nil
		end
	end
end

function Game:delid(id)
	self.gameloop[id]=nil
end

function Game:bint(bool) -- Convert Boolean to Integer
	local result = 0
	if bool then result = 1 end
	return result
end

function Game:Timer(time)
	local Timer = {}
	Timer.maxcount = time
	Timer.timer = time
	Timer.loops = 0

	function Timer:tick(advancement)
		local result = false
		self.timer = self.timer - advancement
		if self.timer<0 then
			result = true
			self.timer = self.timer + self.maxcount
			self.loops = self.loops + 1
		end
		return result
	end

	function Timer:getratio()
		local result = self.timer/self.maxcount
		if result<0 then result = 0 end
		return result
	end

	function Timer:reset()
		self.timer = self.maxcount
		self.loops = 0
	end

	return Timer

end

function Game:flip(canvas,flipx,flipy)
	local c = lg.newCanvas(canvas:getWidth(),canvas:getHeight())
	local xoff = canvas:getWidth()/2
	local yoff = canvas:getHeight()/2
	local xscale = 1
	local yscale = 1
	if flipx then
		xscale = -1
	end
	if flipy then
		yscale = -1
	end
	self:setCanvas(c)
	lg.draw(canvas,xoff,yoff,0,xscale,yscale,xoff,yoff)
	self:setCanvas()
	return c
end

Game.fontwidth = 9
Game.fontheight = 8
function Game:getchar(char)
	local chars = "~!*+,-./0123\"456789:;<=#>?@ABCDEFG$HIJKLMNOPQ%RSTUVWXYZ[&\\]^_`'(){|}§"
	local image = self:newImage("fontmap.png")
	local charw = self.fontwidth
	local charh = self.fontheight
	local index = 1
	local test
	for test in chars:gmatch("[%z\1-\127\194-\244][\128-\191]*") do
		if test==char then break end
		index = index + 1
	end
	local c = lg.newCanvas(charw,charh)
	local nbonline = math.floor(image:getWidth()/charw)
	local x = (index-1)%nbonline
	local y = math.floor((index-1)/nbonline)
	self:setCanvas(c)
	lg.clear()
	if index <= #chars then
		lg.draw(image,-x*charw,-y*charh)
	end
	self:setCanvas()
	return c
end

function Game:getchars(chars)
	local char
	local result = {}
	local line = {}
	for char in chars:gmatch("[%z\1-\127\194-\244][\128-\191]*") do
		if char=="\n" then
			table.insert(result,line)
			line = {}
		else
			table.insert(line,self:getchar(char))
		end
	end
	if #line>0 then table.insert(result,line) end
	return result
end

function Game:setCanvas(c)
	if not c then lg.setCanvas(self.maincanvas) else lg.setCanvas(c) end
end

function Game:playbgm(bgmname,force)
	if self.currentmusic~=bgmname or force then
		if self.currentmusic then self.sounds[self.currentmusic]:stop() end
		self.currentmusic = bgmname
		self.sounds[bgmname]:play()
		self.sounds[bgmname]:setVolume(self.currentvolume*self.bgmvolume)
	end
end

function Game:setVolume(volume)
	self.currentvolume = volume
	if self.currentmusic then
		self.sounds[self.currentmusic]:setVolume(volume*self.bgmvolume)
	end
end

Game:reinit()

return Game
