local game = require "game"

game.OS = love.system.getOS()
game.OSTYPE = "PC"
if game.OS == 'Horizon' then
	game.OSTYPE = love._console_name -- 3DS or Switch
end
if game.OS == "Android" or game.OS == "iOS" then game.OSTYPE = "Mobile" end

lg = love.graphics

function love.load()
	lg.setBackgroundColor(41/255,40/255,49/255)

	if pcall (lg.set3D, true) == true then
		lg.set3D(true)
	end
	lg.setDefaultFilter("nearest")
	game:changesize(416,234)
	love.window.setTitle('Untitled')
	w,h = love.window.getDesktopDimensions()
	love.window.setMode( 416, 234, {fullscreen=true,resizable=true})
	game.shaders.palettemapping:send( "palette_colors", {140/255, 143/255, 174/255}, {88/255, 69/255, 99/255}, {62/255,33/255,55/255}, {154/255,99/255,72/255}, {215/255,155/255,125/255}, {245/255,237/255,186/255}, {192/255,199/255,65/255}, {100/255,125/255,52/255}, {228/255,148/255,58/255}, {157/255,48/255,59/255}, {210/255,100/255,113/255},{112/255,55/255,127/255},{126/255,196/255,193/255},{52/255,133/255,157/255},{23/255,67/255,75/255},{31/255,14/255,28/255} )
	game.maincanvas = lg.newCanvas(game.WIDTH,game.HEIGHT)
end

function love.draw(screen)
	lg.setCanvas(game.maincanvas)
	lg.clear(31/255,14/255,28/255)
	game:draw(screen)
	lg.setCanvas()
	lg.clear(31/255,14/255,28/255)
	lg.setShader(game.shaders.palettemapping)
	lg.draw(game.maincanvas,game.DISPLAYOFFX,game.DISPLAYOFFY,0,game.DISPLAYSCALE,game.DISPLAYSCALE)
	lg.setShader()
end

function love.update()
	print("GAMELOOP;"..game.maxobjects)
	dt = love.timer.getDelta()
	game:step(dt)
	if game.scene then
		game.scenes[game.scene](game,game.sceneargs)
		game.scene = nil
		game.sceneargs = {}
	end
end

function love.keypressed(key,scancode)
	if scancode=="f1" then debug.debug() end
	game.inputs.keys[scancode] = true
end
function love.gamepadpressed(joystick, button) game.inputs.buttons[button] = true
end

function love.keyreleased(key,scancode)
	game.inputs.keys[scancode] = false
end

function love.gamepadreleased(joystick, button)
    game.inputs.buttons[button] = false
end

function love.gamepadaxis( joystick, axis, value )
	game.inputs.axis[axis] = value
end

function love.resize(w, h)
	local scale = math.floor(w/game.WIDTH)
	scale = math.min(scale,math.floor(h/game.HEIGHT))
	local offx = math.floor((w-game.WIDTH*scale)/2)
	local offy = math.floor((h-game.HEIGHT*scale)/2)
	game.DISPLAYSCALE = scale
	game.DISPLAYOFFX = offx
	game.DISPLAYOFFY = offy
end

function love.mousepressed(x,y,button,istouch,presses)
	-- Format positions according to screen
	-- Mobile is handled via the love.touchpressed callback
	if not istouch and x~=nil and y~=nil then
		game:MouseCallback(x,y,-1)
	end
end

function love.mousereleased(x,y,button,istouch,presses)
	-- Format positions according to screen
	-- Mobile is handled via the love.touchreleased callback
	if game.OSTYPE~= "Mobile" and x~=nil and y~=nil then
		game:ReleaseCallback(x,y,-1)
	end
end

function love.touchpressed( id, x, y, dx, dy, pressure )
	-- Format positions according to screen
	if x~=nil and y~=nil then
		game:MouseCallback(x,y,id)
	end
end

function love.touchreleased( id, x, y, dx, dy, pressure )
	-- Format positions according to screen
	if x~=nil and y~=nil then
		game:ReleaseCallback(x,y,id)
	end
end
