local gen = {}

function gen:new(game,args)

	local Breakable = game.objects.base:new(game,args.x,args.y,args.width,args.height)
	Breakable.sprite = game:newImage("breakables/square.png")
	if Breakable.rect.w==32 and Breakable.rect.h == 32 then Breakable.sprite = game:newImage("breakables/big.png") end
	Breakable.tileset = game:findName("Tileset")[1]
	table.insert(Breakable.tileset.solids,Breakable.rect)
	Breakable.rect.disabled = false
	Breakable:register("Breakable")
	
	function Breakable:isOnScreen(rect)
		if not rect then rect = self.rect end
		local margin = 16
		return rect:colliderect(game.rect:new(self.game.camerax-margin,self.game.cameray-margin,self.game.WIDTH+margin,self.game.HEIGHT+margin))
	end

	function Breakable:step(dt)
		if not self.rect.disabled then
			local i
			local v
			for i,v in ipairs(self.game:findName("Player")) do
				if v.rect:colliderect(self.rect,0,2) and v.fastfall then
					self:destroy()
				end
			end
			local i
			local v
			for i,v in ipairs(self.game:findName("Apple")) do
				if v.rect.center.y<self.rect.down and v.rect.center.y>self.rect.up and math.abs(self.rect.center.x-v.rect.center.x)<30 then
					self:destroy()
				end
			end
		else
			if not self:isOnScreen() then self.rect.disabled = false end
		end
	end

	function Breakable:destroy()
		self.rect.disabled = true
		local ox
		local oy
		for ox=1,math.floor(self.rect.w/16) do
			for oy=1,math.floor(self.rect.h/16) do
				local p = self.game.objects.particle:new(game,self.rect.x+(ox-1)*16+8,self.rect.y+(oy-1)*16+8,"particles/spinningdust/",0,0,0,0,10)
				p.depth = 3
			end
		end
		for i=1,5*self.game.particles*self.rect.w*self.rect.h/(16*16) do
			local randx = math.random(-2,2)
			local randy = math.random(-2,2)
			local randt = math.random(0,2)/4
			local randmx = math.random(-30,30)
			local randmy = math.random(0,50)
			self.game.objects.particle:new(self.game,self.rect.center.x+randx,self.rect.center.y+randy,"particles/chunks/",randmx,-100-randmy,0,400,0.2)
		end
	end

	function Breakable:draw()
		if not self.rect.disabled then
			lg.draw(self.sprite,self.rect.x-self.game.camerax,self.rect.y-self.game.cameray)
		end
	end

	return Breakable
end

return gen
