local playergen = {}

function playergen:new(game,args)

	local x = args.x
	local y = args.y+args.height-12
	local Player = game.objects.movable:new(game,x,y,12,12)

	Player.spawnx = x
	Player.spawny = y

	Player:register("GravityAffected")
	Player:register("Player")
	Player.spriteoffx = 2
	Player.spriteoffy = 4
	Player.depth = 1
	game:setcamera(Player.rect.center.x-game.WIDTH/2,Player.rect.center.y-game.HEIGHT/2)
	Player.sprites = game:getSpriteDir("player/walking/")
	Player.sleepsprites = game:getSpriteDir("player/sleep/")
	Player.idlesprites = game:getSpriteDir("player/idle/")
	Player.animtimer = game:Timer(#Player.sprites)
	Player.jumpingsprite = game:newImage("player/jumping.png")
	Player.fallingsprite = game:newImage("player/falling.png")
	Player.landingsprite = game:newImage("player/landing.png")
	Player.hurtsprite = game:newImage("player/hurt.png")
	Player.landingtimer = game:Timer(0.2)
	Player.particletimer = game:Timer(0.03/game.particles)
	Player.bubbletimer = game:Timer(0.03/game.particles)
	Player.moved = false
	Player.flipx = 1

	Player.hknock = 0
	Player.vknock = 0
	Player.reduceknock = 600

	Player.basespeed = 110
	Player.speed = Player.basespeed
	Player.currentspeed = 0
	Player.speedrate = 400
	Player.wind = 0

	Player.invincible = False
	Player.invincibletimer = game:Timer(0.7)

	Player.canfastfall = true
	Player.fastfall = false

	Player.leaptimer = game:Timer(0.3)
	Player.leapimmune = game:Timer(0.3)

	Player.idletimer = game:Timer(0.5)
	Player.sleeploops = 20

	Player.gravity = game.gravity
	Player.maxgravity = 500
	Player.jump = -250
	Player.verspd = 0
	Player.gravityway = 1
	Player.orientation = 1
	Player.switchratio = 0.7

	Player.coyotetimer = game:Timer(0.15)
	Player.coyotetimer:tick(999)

	Player.onground = false
	Player.leftwall = false
	Player.rightwall = false
	Player.drawairborne = false

	function Player:checkJump()
		result = false
		if self.game.inputs.actions.up.lastpress<0.2 or self.game.inputs.actions.jump.lastpress<0.2 then
			result = true
			self.game.inputs.actions.up.lastpress = 99
			self.game.inputs.actions.jump.lastpress = 99
			self.leaptimer:reset()
		end
		return result
	end

	function Player:yeet(hor,ver,noinv)
		if not self.invincible or noinv then
			self.hknock = hor
			self.verspd = ver
			if not noinv then self.currentspeed = 0 end
			local i=1
			for i=1,15 do
				local randx = math.random(-8,8)
				local randy = math.random(-8,8)
				local randt = math.random(-3,3)
				local randmx = math.random(-10,10)
				self.game.objects.particle:new(self.game,self.rect.center.x+randx,self.rect.center.y+randy,"particles/dust/",randmx,-35,hor/3+randy,ver/3+randx,12+randt)
			end
			if noinv then
				self.canfastfall = true
			else
				self.invincible = true
				self.invincibletimer:reset()
			end
		end
	end

	function Player:stillstate(dt) end

	function Player:basestate(dt)
		local left = self.game:bint(self.game.inputs.actions.left.timer>0)
		local right = self.game:bint(self.game.inputs.actions.right.timer>0)
		local down = self.game:bint(self.game.inputs.actions.down.timer>0)
		local up = self.game:bint(self.game.inputs.actions.up.timer>0)

		if self.invincible then
			if self.invincibletimer:tick(dt) then self.invincible = false end
		end

		self.leapimmune:tick(dt)
		if self:inWater(0,0) then
			if self.gravityway > 0 then
				self.gravityway = -1
				if self.leapimmune.loops>1 then self.verspd = self.verspd*self.switchratio end
			end
		else
			if self.gravityway < 0 then
				self.gravityway = 1
				if self.leapimmune.loops>1 then self.verspd = self.verspd*self.switchratio end
			end
		end

		self.onground = false
		self.leftwall = false
		self.rightwall = false
		local lastleft = self.game.inputs.actions.left.lastpress
		local lastright = self.game.inputs.actions.right.lastpress
		if self:checkcollisions(-1,0) then
			if (lastleft<0.3 or left==1) then
				self.leftwall = true
			end
			if self.hknock<0 then self.hknock = 0 end
		end
		if self:checkcollisions(1,0) then
			if (lastright<0.3 or right==1) then
				self.rightwall = true
			end
			if self.hknock>0 then self.hknock = 0 end
		end
		self.airborne = not self:checkcollisions(0,self.gravityway*2,true) or math.abs(self.verspd)>self.gravity/10
		self.orientation = self.gravityway
		if self:checkcollisions(0,-self.gravityway,true) then
			if self.gravityway*self.verspd<=0 then self.verspd = 0 end
			if self.vknock*-self.gravityway>=0 then self.vknock = 0 end
		end
		if self.verspd*self.gravityway>=0 and self:checkcollisions(0,self.gravityway,true) then
			self.onground = true
			self.canfastfall = true
			self.fastfall = false
			self.coyotetimer:reset()
			self.coyotetimer.loops = 0
			if self.vknock*self.gravityway>=0 then self.vknock=0 end
			if self.verspd*self.gravityway>0 then
				if self.airborne then
					self.landingtimer.loops = 0
					-- Landing particles
					for i=1,5*self.game.particles do
						local randx = math.random(-2,2)
						local randy = math.random(-2,2) self.game.objects.particle:new(self.game,self.rect.center.x+randx-self.rect.w/2,self.rect.center.y+randy+self.rect.h/2*self.gravityway,"particles/dust/",-60,self.gravityway*-20,0,0,20)
						self.game.objects.particle:new(self.game,self.rect.center.x+randx+self.rect.w/2,self.rect.center.y+randy+self.rect.h/2*self.gravityway,"particles/dust/",60,self.gravityway*-20,0,0,20)
					end
				end
				self.verspd = 0
			end
			self.leaptimer:reset()
		else
			-- Check for leaps and dives
			if self:inWater(0,10) and not self:inWater(0,-40) then
				self.leaptimer:tick(dt)
				if self.leaptimer.loops>=1 then
					self.floaty = math.min(math.floor(self.rect.y+0.5),self.floaty)
					if self:checkJump() and not (self.leftwall or self.rightwall) then
						self.verspd = self.jump
						self.gravityway = 1
						self.leapimmune:reset()
						self.leapimmune.loops = 0
						-- Jump particles
						for i=1,20*self.game.particles do
							local randx = math.random(-2,2)
							local randy = math.random(-2,2)
							local randt = math.random(-5,5)
							local randmx = math.random(-10,10)
							self.game.objects.particle:new(self.game,self.rect.center.x+randx-self.rect.w/4,self.rect.center.y+randy+self.rect.h/2*-self.gravityway,"particles/dust/",randmx,self.gravityway*30,0,0,15+randt)
						end
					end
					self.orientation = 1
					self.canfastfall = true
				end
			else
				self.leaptimer:reset()
				self.floaty = 999999999
			end
			local gravrate = 1
			if self.verspd*self.gravityway>0 then gravrate = gravrate * 1.5 end
			if math.abs(self.vknock)>10 then gravrate = gravrate * 0.2 end
			self.verspd = self.verspd + self.gravity * dt * self.gravityway * gravrate
			if self.verspd*self.gravityway > 0 then
				if self.leftwall or self.rightwall then
					self.verspd = math.min(math.abs(self.verspd),self.maxgravity/5)*self.gravityway
				end
			end
			self.verspd = math.max(-self.maxgravity,math.min(self.maxgravity,self.verspd))
			self.landingtimer.loops = 1
			if self.game.inputs.actions.stomp.timer==1 and self.canfastfall==true then
				self.fastfall = true
				self.canfastfall = false
				self.verspd = self.maxgravity*self.orientation
				if self.leaptimer.loops>=1 then self.verspd = self.verspd*self.switchratio end
				self.leaptimer:reset()
				self.leapimmune:reset()
				self.leapimmune.loops=0
			end
		end
		if self:checkcollisions(0,self.gravityway,true,true,true) and not self:checkcollisions(0,self.gravityway) then
			-- Falltrough
			if self.game.inputs.actions.down.timer==1 or self.game.inputs.actions.down.realtimer > 0.1 or self.game.inputs.actions.stomp.timer==1 or self.game.inputs.actions.stomp.realtimer > 0.1 then
				self.rect:move(0,1)
			end
		end
		--print("PLAYER HALFWAY CHECKPOINT 1 :DDDD")
		if self.fastfall then
			if self.particletimer:tick(dt) then
				for i=1,2 do
					local randx = math.random(-2,2)
					local randy = math.random(-2,2)
					local randt = math.random(-5,5)
					local randmx = math.random(-10,10)
					local sprite = "particles/dust/"
					if self:inWater(0,0) then sprite = "particles/bubble/" end
					local dir = 1
					if self.verspd < 0 then dir = -1 end
					self.game.objects.particle:new(self.game,self.rect.center.x+randx-self.rect.w/4,self.rect.center.y+randy+self.rect.h/2*dir,sprite,randmx,self.gravityway*-60,0,0,15+randt)
				end
			end
		end
		if math.abs(self.verspd)<50 then self.fastfall = false end
		if self.landingtimer.loops==0 then self.landingtimer:tick(dt) end
		self.coyotetimer:tick(dt)
		if not self.onground and (self.leftwall or self.rightwall) and self.leaptimer.loops<=0 and self:checkJump() then
			-- Walljump
			local hor = -1
			if self.leftwall then hor = 1 end
			self:yeet(hor*300,0,true)
			self.verspd = self.jump*self.gravityway
		end
		if self.coyotetimer.loops==0 then
			local willjump = true -- Check for doors
			if self.onground and math.abs(self.currentspeed)<self.basespeed*0.8 then
				local i
				local j
				for i,v in ipairs(self.game:findName("Door")) do
					if v.rect:collidepoint(self.rect.center.x,self.rect.center.y) then
						willjump = false
					end
				end
			end
			if self:checkJump() and willjump then
				self.coyotetimer:tick(999)
				self.verspd = self.jump * self.gravityway
				-- Jump particles
				for i=1,20*self.game.particles do
					local randx = math.random(-2,2)
					local randy = math.random(-2,2)
					local randt = math.random(-5,5)
					local randmx = math.random(-10,10)
					local sprite = "particles/dust/"
					if self:inWater(0,0) then sprite = "particles/bubble/" end
					self.game.objects.particle:new(self.game,self.rect.center.x+randx-self.rect.w/4,self.rect.center.y+randy+self.rect.h/2*-self.gravityway,sprite,randmx,self.gravityway*30,0,0,15+randt)
				end
			end
		end

		if right==1 then self.flipx = 1 end
		if left==1 then self.flipx = -1 end
		self.speed = self.basespeed
		self.currentspeed = self.currentspeed + (right-left)*dt*self.speedrate*3/2
		if (right-left)==0 then
			local step = dt*self.speedrate*3/2
			if math.abs(self.currentspeed)>step then
			if self.currentspeed > 0 then self.currentspeed = self.currentspeed-step end
			if self.currentspeed < 0 then self.currentspeed = self.currentspeed+step end
			else
				self.currentspeed = 0
			end
		elseif self.onground then
			if self.particletimer:tick(dt) then
				local randx = math.random(-2,2)
				local randy = math.random(-2,2)
				local sprite = "particles/dust/"
				if self:inWater(0,0) then sprite = "particles/bubble/" end
				self.game.objects.particle:new(self.game,self.rect.center.x+randx+self.rect.w/2*(left-right),self.rect.center.y+randy+self.rect.h/2*self.gravityway,sprite,(left-right)*30,self.gravityway*-20,0,0,30)
			end
		end
		print("PLAYER HALFWAY CHECKPOINT :DDDD")
		-- Knockback reduce
		if math.abs(self.hknock)>self.reduceknock*dt then
			if self.hknock < 0 then self.hknock=self.hknock+self.reduceknock*dt end
			if self.hknock > 0 then self.hknock=self.hknock-self.reduceknock*dt end
		else self.hknock=0 end
		if math.abs(self.vknock)>self.reduceknock*dt then
			if self.vknock < 0 then self.vknock=self.vknock+self.reduceknock*dt end
			if self.vknock > 0 then self.vknock=self.vknock-self.reduceknock*dt end
		else self.vknock=0 end
		self.currentspeed = math.max(-self.speed,math.min(self.currentspeed,self.speed))
		self.moved = self:move((self.currentspeed+self.hknock+self.wind)*dt,(self.verspd+self.vknock)*dt,true)
		if up+down+left+right==0 and self.game.inputs.actions.jump.timer==0 then
			if self.idletimer.loops<self.sleeploops then
				self.idletimer:tick(dt*1.5)
			else
				if self.idletimer:tick(dt) then -- spawn Zs
					self.game.objects.particle:new(game,self.rect.right+1,self.rect.center.y+(self.rect.h/2+1)*self.gravityway*-1,"particles/sleep/",20,0,-30,-20)
				end
			end
		else
			self.idletimer:reset()
			self.idletimer.loops = 0
		end
		self.animtimer:tick(dt*math.abs(self.currentspeed)/3)
		local objectivex = self.rect.center.x-self.game.WIDTH/2
		local objectivey = self.rect.center.y-self.game.HEIGHT/2
		if self.moved then
			objectivex = objectivex + self.currentspeed + self.wind
			objectivey = objectivey + self.verspd/4
		end
		local camerarate = 1.5
		local movx = (objectivex-self.game.camerax)*dt*camerarate
		local movy = (objectivey-self.game.cameray)*dt*camerarate
		self.game:movecamera(movx,0)
		if self.leaptimer.loops<1 then
			self.game:movecamera(0,movy)
		end
		local maxy = self.tileset.map.height*16
		if self.tileset.map.height<16 then
			maxy = 0
		end
		self.game:cameraclamp(0,0,self.tileset.map.width*16,maxy)

		-- Spawning particles underwater
		if self:inWater(0,0) then
			local rate = math.sqrt(math.abs(self.verspd)+math.abs(self.currentspeed))
			if self.bubbletimer:tick(dt*0.1+dt*math.sqrt(math.abs(self.verspd)+math.abs(self.currentspeed))*0.02) then
				local i=1
				for i=1,2 do
					local randx = math.random(-10,10)
					local randy = math.random(-3,3)
					local randt = math.random(-5,5)
					local randmx = math.random(-10,10)
					self.game.objects.particle:new(self.game,self.rect.center.x+randx-self.rect.w/4,self.rect.center.y+randy+self.rect.h/2*-self.gravityway,"particles/bubble/",randmx,-35,0,0,10+randt)
				end
			end
		end
		if self:checkspikes() then
			local args = {}
			args.level = self.tileset.mapname
			args.playerspawnid = self.tileset.playerspawnid
			self.state = self.stillstate
			game.objects.transition:new(game,0.3,"main",args)
		end
	end

	Player.state = Player.basestate
	function Player:step(dt)
		self:state(dt)
	end

	function Player:draw()
		local sprite = self.sprites[1+self.animtimer.loops%#self.sprites]
		if math.abs(self.currentspeed) < 2 and self.verspd==0 then
			sprite = self.idlesprites[self.idletimer.loops%#self.idlesprites+1]
			if self.idletimer.loops>self.sleeploops then
				sprite = self.sleepsprites[self.idletimer.loops%#self.sleepsprites+1]
			end
			if self.game.inputs.actions.down.timer>0 then
				sprite = self.landingsprite
			end
		end
		if self.landingtimer.loops==0 then sprite = self.landingsprite end
		local flipx = self.flipx
		if self.airborne then
			local gdir = self.verspd*self.gravityway
			if gdir>0 then sprite = self.fallingsprite end
			if gdir<0 then sprite = self.jumpingsprite end
			if math.abs(self.hknock)+math.abs(self.vknock)>20 and self.invincible then
				sprite = self.hurtsprite
				flipx = -1
				if self.hknock<0 then flipx = 1 end
			end
		end
		local offx = 0
		local offy = 0
		local scaley = 1
		if self.orientation<0 then
			offy = sprite:getHeight()+4
			scaley = -1
		end
		local drawy = math.floor(self.rect.y+0.5)
		if self.leaptimer.loops>1 then
			sprite = self.fallingsprite
			drawy = self.floaty+3
			if self.leaptimer.loops%2==0 then sprite = self.jumpingsprite end
			offy = offy + math.floor(math.sin((1-self.leaptimer:getratio()+self.leaptimer.loops-1)*3)+0.5)
		end
		if flipx==-1 then offx = math.floor(sprite:getWidth()) end
		if self.invincible then
			local rate = 1-math.pow(self.invincibletimer:getratio(),7)
			lg.setColor(1,rate,rate)
		end
		love.graphics.draw(sprite,self.rect.x-self.spriteoffx-self.game.camerax,drawy-self.spriteoffy-game.cameray,0,flipx,scaley,offx,offy)
		lg.setColor(1,1,1)
	end

	return Player

end

return playergen
